@@ -19,17 +19,22 @@
  */
 
 #include "box_data_details.h"
+#include "../frame_glom.h" //For show_ok_dialog().
 #include <glom/libglom/data_structure/field.h>
 #include <glom/libglom/data_structure/relationship.h>
 #include <glom/libglom/data_structure/glomconversions.h>
 #include "dialog_layout_details.h"
 #include <glom/libglom/utils.h>
+#include <glom/glom_privs.h>
 #include "../xsl_utils.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include "../python_embed/glom_python.h"
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 : m_HBox(false, 6),
   m_Button_New(Gtk::Stock::ADD),
@@ -134,6 +139,11 @@ Gnome::Gda::Value Box_Data_Details::get_primary_key_value() const
   return m_primary_key_value;
 }
 
+void Box_Data_Details::set_primary_key_value(const Gtk::TreeModel::iterator& /* row */, const Gnome::Gda::Value& value)
+{
+  m_primary_key_value = value;
+}
+
 bool Box_Data_Details::init_db_details(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value)
 {
   //std::cout << "Box_Data_Details::init_db_details(): primary_key_value.to_string()=" << primary_key_value.to_string() << std::endl;
@@ -190,10 +200,16 @@ bool Box_Data_Details::fill_from_database()
 
   Bakery::BusyCursor busy_cursor(get_app_window());
 
-  const bool primary_key_is_empty = GlomConversions::value_is_empty(m_primary_key_value);
+  const bool primary_key_is_empty = Conversions::value_is_empty(m_primary_key_value);
   if(!primary_key_is_empty)
     get_document()->set_layout_record_viewed(m_table_name, m_layout_name, m_primary_key_value);
 
+  if(!m_field_primary_key)
+  {
+    //refresh_data_from_database_blank(); //shows blank record
+    return false;
+  }
+
   try
   {
 
@@ -210,7 +226,7 @@ bool Box_Data_Details::fill_from_database()
     if(!fieldsToGet.empty())
     {
       //Do not try to show the data if the user may not view it:
-      Privileges table_privs = get_current_privs(m_table_name);
+      Privileges table_privs = Privs::get_current_privs(m_table_name);
 
       //Enable/Disable record creation and deletion:
       m_Button_New.set_sensitive(table_privs.m_create);
@@ -226,11 +242,11 @@ bool Box_Data_Details::fill_from_database()
         //g_warning("primary_key name = %s", m_field_primary_key->get_name().c_str());
         const int index_primary_key = fieldsToGet.size() - 1;
 
-        const Glib::ustring query = GlomUtils::build_sql_select_with_key(m_table_name, fieldsToGet, m_field_primary_key, m_primary_key_value);
+        const Glib::ustring query = Utils::build_sql_select_with_key(m_table_name, fieldsToGet, m_field_primary_key, m_primary_key_value);
         Glib::RefPtr<Gnome::Gda::DataModel> result;
 
         if(!primary_key_is_empty)
-          result = Query_execute(query, get_app_window());
+          result = query_execute(query, get_app_window());
 
         if((result && result->get_n_rows()) || primary_key_is_empty) //either a working result or no result needed.
         {
@@ -266,7 +282,7 @@ bool Box_Data_Details::fill_from_database()
                 value = result->get_value_at(i, row_number);
               else
               {
-                value = GlomConversions::get_empty_value(layout_item->get_glom_type());
+                value = Conversions::get_empty_value(layout_item->get_glom_type());
               }
 
               m_FlowTable.set_field_value(layout_item, value);
@@ -300,7 +316,7 @@ void Box_Data_Details::on_button_new()
 {
   if(confirm_discard_unstored_data())
   {
-    if(m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key && m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
     {
       //Just make a new record, and show it:
       Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_table_name, m_field_primary_key->get_name()); //TODO: This should return a Gda::Value
@@ -320,7 +336,7 @@ void Box_Data_Details::on_button_new()
 
 void Box_Data_Details::on_button_del()
 {
-  if( GlomConversions::value_is_empty(get_primary_key_value()) )
+  if( Conversions::value_is_empty(get_primary_key_value()) )
   {
     //Tell user that a primary key is needed to delete a record:
     Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("No primary key value.")), true);
@@ -392,6 +408,7 @@ void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustrin
 {
   m_FieldsCalculationInProgress.clear();
 
+  //Check all fields in the parent table:
   const Gnome::Gda::Value primary_key_value = get_primary_key_value();
   for(type_vecFields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
@@ -405,7 +422,9 @@ void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustrin
       sharedptr<Field> field = *iter;
       if(field)
       {
-        FieldInRecord field_in_record(m_table_name, field, m_field_primary_key, primary_key_value);
+        sharedptr<LayoutItem_Field> layoutitem_field = sharedptr<LayoutItem_Field>::create();
+        layoutitem_field->set_full_field_details(field);
+        LayoutFieldInRecord field_in_record(layoutitem_field, m_table_name, m_field_primary_key, primary_key_value);
         calculate_field(field_in_record); //And any dependencies.
 
         //Calculate anything that depends on this.
@@ -501,7 +520,7 @@ void Box_Data_Details::on_flowtable_layout_changed()
 
 void Box_Data_Details::on_flowtable_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
 {
-  if(GlomConversions::value_is_empty(primary_key_value))
+  if(Conversions::value_is_empty(primary_key_value))
     return; //Ignore empty ID fields.
 
   signal_requested_related_details().emit(table_name, primary_key_value);
@@ -514,7 +533,7 @@ void Box_Data_Details::on_flowtable_related_record_changed(const Glib::ustring&
 
 void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& field_value)
 {
-  if(GlomConversions::value_is_empty(field_value))
+  if(Conversions::value_is_empty(field_value))
     return; //Ignore empty ID fields.
 
   sharedptr<Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layout_field->get_name());
@@ -529,10 +548,16 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
   if(layout_item)
   {
     const Gnome::Gda::Value primary_key_value = get_primary_key_value();
-    const type_map_fields field_values = get_record_field_values(m_table_name, m_field_primary_key, primary_key_value);
+    const type_map_fields field_values = get_record_field_values_for_calculation(m_table_name, m_field_primary_key, primary_key_value);
+
+   //We need the connection when we run the script, so that the script may use it.
+   sharedptr<SharedConnection> sharedconnection = connect_to_server(0 /* parent window */);
 
     glom_execute_python_function_implementation(layout_item->get_script(), field_values, //TODO: Maybe use the field's type here.
-    get_document(), get_table_name());
+    get_document(), get_table_name(), sharedconnection->get_gda_connection());
+
+    //Refresh the view, in case the script changed any data:
+    refresh_data_from_database_with_primary_key(m_primary_key_value);
   }
 }
 
@@ -543,8 +568,12 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
 
   const Glib::ustring strFieldName = layout_field->get_name();
 
+  Gtk::Window* window = get_app_window();
+
+  Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+
   Gnome::Gda::Value primary_key_value = get_primary_key_value();
-  if(!GlomConversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
+  if(!Conversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
   {
     Glib::ustring table_name;
     sharedptr<Field> primary_key_field;
@@ -562,8 +591,6 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
       //plus how to identify the record in that table.
       const Glib::ustring relationship_name = layout_field->get_relationship_name();
 
-      Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
-
       sharedptr<Relationship> relationship = document->get_relationship(get_table_name(), relationship_name);
       if(relationship)
       {
@@ -595,23 +622,28 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
       }
     }
 
-    //const sharedptr<const Field>& field = layout_field->m_field;
+
+    LayoutFieldInRecord field_in_record(layout_field, m_table_name /* parent table */, primary_key_field, primary_key_value);
+
+    //Check whether the value meets uniqueness constraints:
+    if(!check_entered_value_for_uniqueness(m_table_name, layout_field, field_value, window))
+    {
+      //Revert to the value in the database:
+      const Gnome::Gda::Value value_old = get_field_value_in_database(field_in_record, window);
+      set_entered_field_data(layout_field, value_old);
+   
+      return; 
+    }
 
     //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
     //if(layout_field->get_glom_type() != Field::TYPE_IMAGE) //TODO For now, don't do this for images, because the ImageGlom widget expects a broken GdaValue, because gda_value_get_binary() needs to be fixed.
-      m_FlowTable.set_field_value(layout_field, field_value);
+    m_FlowTable.set_field_value(layout_field, field_value);
 
     //Update the field in the record (the record with this primary key):
     try
     {
-      FieldInRecord field_in_record(layout_field, m_table_name /* parent table */, primary_key_field, primary_key_value, *(get_document()));
       const bool bTest = set_field_value_in_database(field_in_record, field_value, false /* don't use current calculations */, get_app_window());
 
-      //Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
-      //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
-      //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
-      //bool bTest = Query_execute(strQuery);
-
       if(!bTest)
       {
         //Update failed.
@@ -651,7 +683,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
   {
     //There is no current primary key value:
 
-    if(m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key && m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
     {
       if(strFieldName == m_field_primary_key->get_name()) //If edited field is the primary key.
       {
@@ -674,12 +706,21 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     {
       //It is not auto-generated:
 
-      if(strFieldName == m_field_primary_key->get_name()) //if it is the primary key that' being edited.
+      if(strFieldName == m_field_primary_key->get_name()) //if it is the primary key that is being edited.
       {
-        //Create new record with this primary key,
-        //and all the other field values too.
-        //see comments after 'else':
-        record_new(true /* use entered field data */);
+        if(!check_entered_value_for_uniqueness(m_table_name, layout_field, field_value, window))
+        {
+          //Revert to a blank value:
+          const Gnome::Gda::Value value_old = Conversions::get_empty_value(layout_field->get_full_field_details()->get_glom_type());
+          set_entered_field_data(layout_field, value_old);
+        }
+        else
+        {
+          //Create new record with this primary key,
+          //and all the other field values too.
+          //see comments after 'else':
+          record_new(true /* use entered field data */);
+        }
       }
       else
       {
@@ -726,7 +767,7 @@ void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const sha
         nodeField->set_attribute("title", pLayoutField->get_title_or_name());
 
         Gnome::Gda::Value value = m_FlowTable.get_field_value(pLayoutField);
-        const Glib::ustring text_representation = GlomConversions::get_text_for_gda_value(pLayoutField->get_glom_type(), value,
+        const Glib::ustring text_representation = Conversions::get_text_for_gda_value(pLayoutField->get_glom_type(), value,
           pLayoutField->get_formatting_used().m_numeric_format); //In the current locale.
 
         nodeField->set_attribute("value", text_representation);
@@ -758,7 +799,7 @@ void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const sha
 
 void Box_Data_Details::print_layout()
 {
-  const Privileges table_privs = get_current_privs(m_table_name);
+  const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
@@ -798,3 +839,5 @@ void Box_Data_Details::print_layout()
     GlomXslUtils::transform_and_open(*pDocument, "print_details_to_html.xsl", get_app_window());
   }
 }
+
+} //namespace Glom
